@@ -62,17 +62,37 @@ Because Fiona collections are context managers, they are closed and (in
 writing modes) flush contents to disk when their ``with`` blocks end.
 """
 
+from contextlib import contextmanager
 import logging
 import os
+import sys
+import warnings
+
 from six import string_types
 
-from fiona.collection import Collection, BytesCollection, vsi_path
-from fiona._drivers import driver_count, GDALEnv
+try:
+    from pathlib import Path
+except ImportError:  # pragma: no cover
+    class Path:
+        pass
+
+if sys.platform == "win32":
+    libdir = os.path.join(os.path.dirname(__file__), ".libs")
+    os.environ["PATH"] = os.environ["PATH"] + ";" + libdir
+
+from fiona.collection import BytesCollection, Collection
 from fiona.drvsupport import supported_drivers
+from fiona.env import ensure_env_with_credentials, Env
+from fiona.errors import FionaDeprecationWarning
+from fiona._env import driver_count
+from fiona._env import (
+    calc_gdal_version_num, get_gdal_version_num, get_gdal_release_name,
+    get_gdal_version_tuple)
 from fiona.compat import OrderedDict
-from fiona.ogrext import _bounds, _listlayers, FIELD_TYPES_MAP, _remove
-from fiona.ogrext import (
-    calc_gdal_version_num, get_gdal_version_num, get_gdal_release_name)
+from fiona.io import MemoryFile
+from fiona.ogrext import _bounds, _listlayers, FIELD_TYPES_MAP, _remove, _remove_layer
+from fiona.path import ParsedPath, parse_path, vsi_path
+from fiona.vfs import parse_paths as vfs_parse_paths
 
 # These modules are imported by fiona.ogrext, but are also import here to
 # help tools like cx_Freeze find them automatically
@@ -81,25 +101,20 @@ import uuid
 
 
 __all__ = ['bounds', 'listlayers', 'open', 'prop_type', 'prop_width']
-__version__ = "1.7.10"
-__gdal_version__ = get_gdal_release_name().decode('utf-8')
+__version__ = "1.8.4"
+__gdal_version__ = get_gdal_release_name()
+
+gdal_version = get_gdal_version_tuple()
 
 log = logging.getLogger(__name__)
+log.addHandler(logging.NullHandler())
 
 
-def open(
-        path,
-        mode='r',
-        driver=None,
-        schema=None,
-        crs=None,
-        encoding=None,
-        layer=None,
-        vfs=None,
-        enabled_drivers=None,
-        crs_wkt=None):
-    """Open file at ``path`` in ``mode`` "r" (read), "a" (append), or
-    "w" (write) and return a ``Collection`` object.
+@ensure_env_with_credentials
+def open(fp, mode='r', driver=None, schema=None, crs=None, encoding=None,
+         layer=None, vfs=None, enabled_drivers=None, crs_wkt=None,
+         **kwargs):
+    """Open a collection for read, append, or write
 
     In write mode, a driver name such as "ESRI Shapefile" or "GPX" (see
     OGR docs or ``ogr2ogr --help`` on the command line) and a schema
@@ -133,11 +148,6 @@ def open(
     When the provided path is to a file containing multiple named layers
     of data, a layer can be singled out by ``layer``.
 
-    A virtual filesystem can be specified. The ``vfs`` parameter may be
-    an Apache Commons VFS style string beginning with "zip://" or
-    "tar://"". In this case, the ``path`` must be an absolute path
-    within that container.
-
     The drivers enabled for opening datasets may be restricted to those
     listed in the ``enabled_drivers`` parameter. This and the ``driver``
     parameter afford much control over opening of files.
@@ -151,37 +161,116 @@ def open(
       fiona.open(
           'example.shp', enabled_drivers=['GeoJSON', 'ESRI Shapefile'])
 
+    Parameters
+    ----------
+    fp : URI (str or pathlib.Path), or file-like object
+        A dataset resource identifier or file object.
+    mode : str
+        One of 'r', to read (the default); 'a', to append; or 'w', to
+        write.
+    driver : str
+        In 'w' mode a format driver name is required. In 'r' or 'a'
+        mode this parameter has no effect.
+    schema : dict
+        Required in 'w' mode, has no effect in 'r' or 'a' mode.
+    crs : str or dict
+        Required in 'w' mode, has no effect in 'r' or 'a' mode.
+    encoding : str
+        Name of the encoding used to encode or decode the dataset.
+    layer : int or str
+        The integer index or name of a layer in a multi-layer dataset.
+    vfs : str
+        This is a deprecated parameter. A URI scheme such as "zip://"
+        should be used instead.
+    enabled_drivers : list
+        An optional list of driver names to used when opening a
+        collection.
+    crs_wkt : str
+        An optional WKT representation of a coordinate reference
+        system.
+    kwargs : mapping
+        Other driver-specific parameters that will be interpreted by
+        the OGR library as layer creation or opening options.
+
+    Returns
+    -------
+    Collection
     """
-    # Parse the vfs into a vsi and an archive path.
-    path, vsi, archive = parse_paths(path, vfs)
-    if mode in ('a', 'r'):
-        if archive:
-            if not os.path.exists(archive):
-                raise IOError("no such archive file: %r" % archive)
-        elif path != '-' and not os.path.exists(path):
-            raise IOError("no such file or directory: %r" % path)
-        c = Collection(path, mode, driver=driver, encoding=encoding,
-                       layer=layer, vsi=vsi, archive=archive,
-                       enabled_drivers=enabled_drivers)
-    elif mode == 'w':
+
+    if mode == 'r' and hasattr(fp, 'read'):
+
+        @contextmanager
+        def fp_reader(fp):
+            memfile = MemoryFile(fp.read())
+            dataset = memfile.open()
+            try:
+                yield dataset
+            finally:
+                dataset.close()
+                memfile.close()
+
+        return fp_reader(fp)
+
+    elif mode == 'w' and hasattr(fp, 'write'):
         if schema:
             # Make an ordered dict of schema properties.
             this_schema = schema.copy()
             this_schema['properties'] = OrderedDict(schema['properties'])
         else:
             this_schema = None
-        c = Collection(path, mode, crs=crs, driver=driver, schema=this_schema,
-                       encoding=encoding, layer=layer, vsi=vsi, archive=archive,
-                       enabled_drivers=enabled_drivers, crs_wkt=crs_wkt)
+
+        @contextmanager
+        def fp_writer(fp):
+            memfile = MemoryFile()
+            dataset = memfile.open(
+                driver=driver, crs=crs, schema=schema, layer=layer,
+                encoding=encoding, enabled_drivers=enabled_drivers,
+                **kwargs)
+            try:
+                yield dataset
+            finally:
+                dataset.close()
+                memfile.seek(0)
+                fp.write(memfile.read())
+                memfile.close()
+
+        return fp_writer(fp)
+
     else:
-        raise ValueError(
-            "mode string must be one of 'r', 'w', or 'a', not %s" % mode)
-    return c
+        # If a pathlib.Path instance is given, convert it to a string path.
+        if isinstance(fp, Path):
+            fp = str(fp)
+
+        if vfs:
+            # Parse the vfs into a vsi and an archive path.
+            path, scheme, archive = vfs_parse_paths(fp, vfs=vfs)
+            path = ParsedPath(path, archive, scheme)
+        else:
+            path = parse_path(fp)
+
+        if mode in ('a', 'r'):
+            c = Collection(path, mode, driver=driver, encoding=encoding,
+                           layer=layer, enabled_drivers=enabled_drivers, **kwargs)
+        elif mode == 'w':
+            if schema:
+                # Make an ordered dict of schema properties.
+                this_schema = schema.copy()
+                this_schema['properties'] = OrderedDict(schema['properties'])
+            else:
+                this_schema = None
+            c = Collection(path, mode, crs=crs, driver=driver, schema=this_schema,
+                           encoding=encoding, layer=layer, enabled_drivers=enabled_drivers, crs_wkt=crs_wkt,
+                           **kwargs)
+        else:
+            raise ValueError(
+                "mode string must be one of 'r', 'w', or 'a', not %s" % mode)
+
+        return c
 
 collection = open
 
 
-def remove(path_or_collection, driver=None):
+def remove(path_or_collection, driver=None, layer=None):
     """Deletes an OGR data source
 
     The required ``path`` argument may be an absolute or relative file path.
@@ -203,11 +292,13 @@ def remove(path_or_collection, driver=None):
         collection.close()
     else:
         path = path_or_collection
-        if driver is None:
-            raise ValueError("The driver argument is required when removing a path")
-    _remove(path, driver)
+    if layer is None:
+        _remove(path, driver)
+    else:
+        _remove_layer(path, layer, driver)
 
 
+@ensure_env_with_credentials
 def listlayers(path, vfs=None):
     """Returns a list of layer names in their index order.
 
@@ -224,29 +315,14 @@ def listlayers(path, vfs=None):
     if vfs and not isinstance(vfs, string_types):
         raise TypeError("invalid vfs: %r" % vfs)
 
-    path, vsi, archive = parse_paths(path, vfs)
-
-    if archive:
-        if not os.path.exists(archive):
-            raise IOError("no such archive file: %r" % archive)
-    elif not os.path.exists(path):
-        raise IOError("no such file or directory: %r" % path)
-
-    with drivers():
-        return _listlayers(vsi_path(path, vsi, archive))
-
-
-def parse_paths(path, vfs=None):
-    archive = vsi = None
     if vfs:
-        parts = vfs.split("://")
-        vsi = parts.pop(0) if parts else None
-        archive = parts.pop(0) if parts else None
+        pobj_vfs = parse_path(vfs)
+        pobj_path = parse_path(path)
+        pobj = ParsedPath(pobj_path.path, pobj_vfs.path, pobj_vfs.scheme)
     else:
-        parts = path.split("://")
-        path = parts.pop() if parts else None
-        vsi = parts.pop() if parts else None
-    return path, vsi, archive
+        pobj = parse_path(path)
+
+    return _listlayers(vsi_path(pobj))
 
 
 def prop_width(val):
@@ -279,13 +355,18 @@ def prop_type(text):
 
 
 def drivers(*args, **kwargs):
-    """Returns a context manager with registered drivers."""
+    """Returns a context manager with registered drivers.
+
+    DEPRECATED
+    """
+    warnings.warn("Use fiona.Env() instead.", FionaDeprecationWarning, stacklevel=2)
+
     if driver_count == 0:
         log.debug("Creating a chief GDALEnv in drivers()")
-        return GDALEnv(**kwargs)
+        return Env(**kwargs)
     else:
         log.debug("Creating a not-responsible GDALEnv in drivers()")
-        return GDALEnv(**kwargs)
+        return Env(**kwargs)
 
 
 def bounds(ob):
