@@ -1,21 +1,27 @@
 # -*- coding: utf-8 -*-
 # Collections provide file-like access to feature data
 
-
+import logging
 import os
+import warnings
 
-from fiona import compat
+from fiona import compat, vfs
 from fiona.ogrext import Iterator, ItemsIterator, KeysIterator
 from fiona.ogrext import Session, WritingSession
-from fiona.ogrext import (
-    calc_gdal_version_num, get_gdal_version_num, get_gdal_release_name)
-from fiona.ogrext import buffer_to_virtual_file, remove_virtual_file
-from fiona.errors import DriverError, SchemaError, CRSError
-from fiona._drivers import driver_count, GDALEnv
+from fiona.ogrext import buffer_to_virtual_file, remove_virtual_file, GEOMETRY_TYPES
+from fiona.errors import (DriverError, SchemaError, CRSError, UnsupportedGeometryTypeError, DriverSupportError)
+from fiona.logutils import FieldSkipLogFilter
+from fiona._env import driver_count, get_gdal_release_name, get_gdal_version_tuple
+from fiona.env import Env
+from fiona.errors import FionaDeprecationWarning
 from fiona.drvsupport import supported_drivers
+from fiona.path import Path, UnparsedPath, vsi_path, parse_path
 from six import string_types, binary_type
 
 
+log = logging.getLogger(__name__)
+
+
 class Collection(object):
 
     """A file-like interface to features of a vector dataset
@@ -27,7 +33,9 @@ class Collection(object):
 
     def __init__(self, path, mode='r', driver=None, schema=None, crs=None,
                  encoding=None, layer=None, vsi=None, archive=None,
-                 enabled_drivers=None, crs_wkt=None, **kwargs):
+                 enabled_drivers=None, crs_wkt=None, ignore_fields=None,
+                 ignore_geometry=False,
+                 **kwargs):
 
         """The required ``path`` is the absolute or relative path to
         a file, such as '/data/test_uk.shp'. In ``mode`` 'r', data can
@@ -44,7 +52,7 @@ class Collection(object):
         options.
         """
 
-        if not isinstance(path, string_types):
+        if not isinstance(path, (string_types, Path)):
             raise TypeError("invalid path: %r" % path)
         if not isinstance(mode, string_types) or mode not in ('r', 'w', 'a'):
             raise TypeError("invalid mode: %r" % mode)
@@ -61,14 +69,13 @@ class Collection(object):
         if layer and not isinstance(layer, tuple(list(string_types) + [int])):
             raise TypeError("invalid name: %r" % layer)
         if vsi:
-            if not isinstance(vsi, string_types) or vsi not in ('zip', 'tar', 'gzip'):
+            if not isinstance(vsi, string_types) or not vfs.valid_vsi(vsi):
                 raise TypeError("invalid vsi: %r" % vsi)
         if archive and not isinstance(archive, string_types):
             raise TypeError("invalid archive: %r" % archive)
 
         # Check GDAL version against drivers
-        if (driver == "GPKG" and
-                get_gdal_version_num() < calc_gdal_version_num(1, 11, 0)):
+        if (driver == "GPKG" and get_gdal_version_tuple() < (1, 11, 0)):
             raise DriverError(
                 "GPKG driver requires GDAL 1.11.0, fiona was compiled "
                 "against: {}".format(get_gdal_release_name()))
@@ -83,19 +90,26 @@ class Collection(object):
         self._crs_wkt = None
         self.env = None
         self.enabled_drivers = enabled_drivers
+        self.ignore_fields = ignore_fields
+        self.ignore_geometry = bool(ignore_geometry)
 
-        self.path = vsi_path(path, vsi, archive)
+        if vsi:
+            self.path = vfs.vsi_path(path, vsi, archive)
+            path = parse_path(self.path)
+        else:
+            path = parse_path(path)
+            self.path = vsi_path(path)
 
         if mode == 'w':
             if layer and not isinstance(layer, string_types):
-                raise ValueError("in 'r' mode, layer names must be strings")
+                raise ValueError("in 'w' mode, layer names must be strings")
             if driver == 'GeoJSON':
                 if layer is not None:
                     raise ValueError("the GeoJSON format does not have layers")
                 self.name = 'OgrGeoJSON'
             # TODO: raise ValueError as above for other single-layer formats.
             else:
-                self.name = layer or os.path.basename(os.path.splitext(path)[0])
+                self.name = layer or os.path.basename(os.path.splitext(path.path)[0])
         else:
             if layer in (0, None):
                 self.name = 0
@@ -125,6 +139,8 @@ class Collection(object):
                 raise SchemaError("schema lacks: geometry")
             self._schema = schema
 
+            self._check_schema_driver_support()
+
             if crs_wkt:
                 self._crs_wkt = crs_wkt
             elif crs:
@@ -133,32 +149,30 @@ class Collection(object):
                 else:
                     raise CRSError("crs lacks init or proj parameter")
 
-        if driver_count == 0:
-            # create a local manager and enter
-            self.env = GDALEnv()
-        else:
-            self.env = GDALEnv()
-        self.env.__enter__()
+        self._driver = driver
+        kwargs.update(encoding=encoding or '')
+        self.encoding = encoding
 
-        if self.mode == "r":
-            self._driver = driver
-            self.encoding = encoding
-            self.session = Session()
-            self.session.start(self)
+        try:
+            if self.mode == 'r':
+                self.session = Session()
+                self.session.start(self, **kwargs)
+            elif self.mode in ('a', 'w'):
+                self.session = WritingSession()
+                self.session.start(self, **kwargs)
+        except IOError:
+            self.session = None
+            raise
 
-            # If encoding param is None, we'll use what the session
-            # suggests.
-            self.encoding = encoding or self.session.get_fileencoding().lower()
-
-        elif self.mode in ("a", "w"):
-            self._driver = driver
-            self.encoding = encoding
-            self.session = WritingSession()
-            self.session.start(self, **kwargs)
-            self.encoding = encoding or self.session.get_fileencoding().lower()
-
-        if self.session:
+        if self.session is not None:
             self.guard_driver_mode()
+            if not self.encoding:
+                self.encoding = self.session.get_fileencoding().lower()
+
+        if self.mode in ("a", "w"):
+            self._valid_geom_types = _get_valid_geom_types(self.schema, self.driver)
+
+        self.field_skip_log_filter = FieldSkipLogFilter()
 
     def __repr__(self):
         return "<%s Collection '%s', mode '%s' at %s>" % (
@@ -311,6 +325,9 @@ class Collection(object):
 
     def __next__(self):
         """Returns next record from iterator."""
+        warnings.warn("Collection.__next__() is buggy and will be removed in "
+                      "Fiona 2.0. Switch to `next(iter(collection))`.",
+                      FionaDeprecationWarning, stacklevel=2)
         if not self.iterator:
             iter(self)
         return next(self.iterator)
@@ -320,6 +337,9 @@ class Collection(object):
     def __getitem__(self, item):
         return self.session.__getitem__(item)
 
+    def get(self, item):
+        return self.session.get(item)
+
     def writerecords(self, records):
         """Stages multiple records for writing to disk."""
         if self.closed:
@@ -381,9 +401,38 @@ class Collection(object):
             self._bounds = self.session.get_extent()
         return self._bounds
 
+    def _check_schema_driver_support(self):
+        """Check support for the schema against the driver
+
+        See GH#572 for discussion.
+        """
+        gdal_version_major = get_gdal_version_tuple().major
+
+        for field in self._schema["properties"].values():
+            field_type = field.split(":")[0]
+            if self._driver == "ESRI Shapefile":
+                if field_type == "datetime":
+                    raise DriverSupportError("ESRI Shapefile does not support datetime fields")
+                elif field_type == "time":
+                    raise DriverSupportError("ESRI Shapefile does not support time fields")
+            elif self._driver == "GPKG":
+                if field_type == "time":
+                    raise DriverSupportError("GPKG does not support time fields")
+                elif gdal_version_major == 1:
+                    if field_type == "datetime":
+                        raise DriverSupportError("GDAL 1.x GPKG driver does not support datetime fields")
+            elif self._driver == "GeoJSON":
+                if gdal_version_major == 1:
+                    if field_type == "date":
+                        warnings.warn("GeoJSON driver in GDAL 1.x silently converts date to string in non-standard format")
+                    elif field_type == "datetime":
+                        warnings.warn("GeoJSON driver in GDAL 1.x silently converts datetime to string in non-standard format")
+                    elif field_type == "time":
+                        warnings.warn("GeoJSON driver in GDAL 1.x silently converts time to string")
+
     def flush(self):
         """Flush the buffer."""
-        if self.session is not None and self.session.get_length() > 0:
+        if self.session is not None:
             self.session.sync(self)
             new_len = self.session.get_length()
             self._len = new_len > self._len and new_len or self._len
@@ -392,10 +441,12 @@ class Collection(object):
     def close(self):
         """In append or write mode, flushes data to disk, then ends
         access."""
-        if self.session is not None:
+        if self.session is not None and self.session.isactive():
             if self.mode in ('a', 'w'):
                 self.flush()
+            log.debug("Flushed buffer")
             self.session.stop()
+            log.debug("Stopped session")
             self.session = None
             self.iterator = None
         if self.env:
@@ -407,22 +458,64 @@ class Collection(object):
         return self.session is None
 
     def __enter__(self):
+        logging.getLogger('fiona.ogrext').addFilter(self.field_skip_log_filter)
         return self
 
     def __exit__(self, type, value, traceback):
+        logging.getLogger('fiona.ogrext').removeFilter(self.field_skip_log_filter)
         self.close()
 
     def __del__(self):
         # Note: you can't count on this being called. Call close() explicitly
         # or use the context manager protocol ("with").
-        self.__exit__(None, None, None)
+        self.close()
+
+
+ALL_GEOMETRY_TYPES = set([
+    geom_type for geom_type in GEOMETRY_TYPES.values()
+    if "3D " not in geom_type and geom_type != "None"])
+ALL_GEOMETRY_TYPES.add("None")
+
+
+def _get_valid_geom_types(schema, driver):
+    """Returns a set of geometry types the schema will accept"""
+    schema_geom_type = schema["geometry"]
+    if isinstance(schema_geom_type, string_types) or schema_geom_type is None:
+        schema_geom_type = (schema_geom_type,)
+    valid_types = set()
+    for geom_type in schema_geom_type:
+        geom_type = str(geom_type).lstrip("3D ")
+        if geom_type == "Unknown" or geom_type == "Any":
+            valid_types.update(ALL_GEOMETRY_TYPES)
+        else:
+            if geom_type not in ALL_GEOMETRY_TYPES:
+                raise UnsupportedGeometryTypeError(geom_type)
+            valid_types.add(geom_type)
+
+    # shapefiles don't differentiate between single/multi geometries, except points
+    if driver == "ESRI Shapefile" and "Point" not in valid_types:
+        for geom_type in list(valid_types):
+            if not geom_type.startswith("Multi"):
+                valid_types.add("Multi" + geom_type)
+
+    return valid_types
+
+
+def get_filetype(bytesbuf):
+    """Detect compression type of bytesbuf.
+
+    ZIP only. TODO: add others relevant to GDAL/OGR."""
+    if bytesbuf[:4].startswith(b'PK\x03\x04'):
+        return 'zip'
+    else:
+        return ''
 
 
 class BytesCollection(Collection):
     """BytesCollection takes a buffer of bytes and maps that to
     a virtual file that can then be opened by fiona.
     """
-    def __init__(self, bytesbuf):
+    def __init__(self, bytesbuf, **kwds):
         """Takes buffer of bytes whose contents is something we'd like
         to open with Fiona and maps it to a virtual file.
         """
@@ -433,11 +526,21 @@ class BytesCollection(Collection):
         # it is garbage collected while in use.
         self.bytesbuf = bytesbuf
 
-        # Map the buffer to a file.
-        self.virtual_file = buffer_to_virtual_file(self.bytesbuf)
+        # Map the buffer to a file. If the buffer contains a zipfile
+        # we take extra steps in naming the buffer and in opening
+        # it. If the requested driver is for GeoJSON, we append an an
+        # appropriate extension to ensure the driver reads it.
+        filetype = get_filetype(self.bytesbuf)
+        ext = ''
+        if filetype == 'zip':
+            ext = '.zip'
+        elif kwds.get('driver') == "GeoJSON":
+            ext = '.json'
+        self.virtual_file = buffer_to_virtual_file(self.bytesbuf, ext=ext)
 
         # Instantiate the parent class.
-        super(BytesCollection, self).__init__(self.virtual_file)
+        super(BytesCollection, self).__init__(self.virtual_file, vsi=filetype,
+                                              encoding='utf-8', **kwds)
 
     def close(self):
         """Removes the virtual file associated with the class."""
@@ -453,16 +556,3 @@ class BytesCollection(Collection):
             self.path + ":" + str(self.name),
             self.mode,
             hex(id(self)))
-
-
-def vsi_path(path, vsi=None, archive=None):
-    # If a VSF and archive file are specified, we convert the path to
-    # an OGR VSI path (see cpl_vsi.h).
-    if vsi:
-        if archive:
-            result = "/vsi%s/%s%s" % (vsi, archive, path)
-        else:
-            result = "/vsi%s/%s" % (vsi, path)
-    else:
-        result = path
-    return result
