@@ -5,7 +5,6 @@ GDALOPTS="  --with-ogr \
             --with-geos \
             --with-expat \
             --without-libtool \
-            --with-libz=internal \
             --with-libtiff=internal \
             --with-geotiff=internal \
             --without-gif \
@@ -33,8 +32,8 @@ GDALOPTS="  --with-ogr \
             --without-ingres \
             --without-xerces \
             --without-odbc \
-            --without-curl \
-            --without-sqlite3 \
+            --with-curl \
+            --with-sqlite3 \
             --without-dwgdirect \
             --without-idb \
             --without-sde \
@@ -54,25 +53,21 @@ fi
 
 ls -l $GDALINST
 
-if [ "$GDALVERSION" = "1.9.2" -a ! -d "$GDALINST/gdal-$GDALVERSION" ]; then
-  cd $GDALBUILD
-  wget http://download.osgeo.org/gdal/gdal-$GDALVERSION.tar.gz
-  tar -xzf gdal-$GDALVERSION.tar.gz
-  cd gdal-$GDALVERSION
+if [ "$GDALVERSION" = "trunk" ]; then
+  # always rebuild trunk
+  git clone -b master --single-branch --depth=1 https://github.com/OSGeo/gdal.git $GDALBUILD/trunk
+  cd $GDALBUILD/trunk/gdal
   ./configure --prefix=$GDALINST/gdal-$GDALVERSION $GDALOPTS
-  make -s -j 2
+  make -j 2
   make install
-fi
-
-
-# download and compile gdal version
-if [ "$GDALVERSION" != "1.9.2" -a ! -d "$GDALINST/gdal-$GDALVERSION" ]; then
+elif [ ! -d "$GDALINST/gdal-$GDALVERSION" ]; then
+  # only build if not already installed
   cd $GDALBUILD
   wget http://download.osgeo.org/gdal/$GDALVERSION/gdal-$GDALVERSION.tar.gz
   tar -xzf gdal-$GDALVERSION.tar.gz
   cd gdal-$GDALVERSION
   ./configure --prefix=$GDALINST/gdal-$GDALVERSION $GDALOPTS
-  make -s -j 2
+  make -j 2
   make install
 fi
 
